/*
* ASIX FORTE I2C analyzer example
* distributed as-is, without any warranties
* http://www.asix.net/
*
* version 1.0, December 2020
* 
* Programmer connection table: 
* I2C  <->  FORTE
* VDD     <->  VDD
* GND     <->  GND
* SDA     <->  D
* SCK     <->  C
*
*/

#include <stdio.h>
#include "forte.h"

void ReadStatus(char* message)
{
    int status = AGetBlocking();
    char* codestring = ReturnCodeString(status);
    if(message[0] != '\0') {
        printf("%s: %s\n", message, codestring);
    }
    else {
        printf("%s\n", codestring);
    }
}

void PrintHex(int* data, int count)
{
    int i;
    for(i=0;i<count;i++) {
        printf("%02X ", data[i]);
        if(((i+1)%16) == 0) printf("\n");
    }
    if((count%16) != 0) printf("\n");
}

unsigned char LsbToMsb(unsigned char databyte)
{
    int i;
    unsigned char x = 0;
    for(i=0;i<8;i++) {
        x = x | (((databyte >> i) & 1) << (7-i));
    }
    return x;
}

int main(int argc, char *argv[])
{
    int i, count, received_bytes;
    int data_array[256];
    int intdata;
    
    printf("ASIX FORTE I2C analyzer example \n");
   
    if(LoadForteDll("forte.dll") == FORTE_DLL_LOAD_OK) {
        printf("DLL load OK\n");
    }
    else {
        printf("DLL load FAILED\n");
        return 1;
    }
    
    QOpenProg(-1);
    ReadStatus("");
    
    // start the analyzer
    QAnI2CStart();
    ReadStatus("");
      
     
    // wait for 100 answers
    printf("Waiting for the data\n");    
    count=100;
    
    i=0;
    while (i<count) {
         // it is possible to wait using AGet or AGetBlocking    
         //  if (AGet(&intdata)!=0)
         intdata=AGetBlocking();
         {
           // received valid data
           if (intdata == (AN_DATA | AN_I2C_ACK)){
              printf("%d, %x, AN_I2C ACK\n", i, intdata);
           }  
           else
           if (intdata == (AN_DATA | AN_I2C_NACK)){
              printf("%d, %x, AN_I2C NACK\n", i, intdata);
           }  
           else
           if (intdata == (AN_DATA | AN_I2C_START)){
              printf("%d, %x, AN_I2C START\n", i, intdata);
           }  
           else
           if (intdata == (AN_DATA | AN_I2C_STOP)){
              printf("%d, %x, AN_I2C STOP\n", i, intdata);
           }  
           else
           if ((intdata & AN_DATA) == AN_DATA){
              printf("%d, %x, AN_I2C %x\n", i, intdata, intdata & AN_I2C_DATAMASK);
           }  
           else {                               
           printf("%d, %x\n", i, intdata);
           }
           
           i++;
         }
    }
    

    printf("%d values read\n", i);

    // we have enough answers, stop the analyzer
    QAnStop();


    // read the rest captured data including the QAnStop answer 
    // ------------  
    received_bytes=1;
    while (received_bytes>0){
         AGetBlock(data_array, 200, &received_bytes);
    }
    // ------------
        
    QCloseProg();
    ReadStatus("");
    
    return 0;
}

