/////////////////////////////////////////////////////////////////////////////
// Name:        device_DS18B20.cpp
// Purpose:     1-wire thermometer
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// standard header file
#include "device_DS18B20.h"

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// class DeviceDS18B20
// ----------------------------------------------------------------------------
// constructor
DeviceDS18B20::DeviceDS18B20()  
{
    //
}

// virtual function override

wxString DeviceDS18B20::GetName()
{
    return _T("DS18B20");
}

wxString DeviceDS18B20::GetShortHint()
{
    return _T("1-wire thermometer");
}

wxString DeviceDS18B20::GetLongHint()
{
    wxString s;
    s += "Device pinout (TO-92 package):\n";
    s += "1 ... GND\n";
    s += "2 ... DQ\n";
    s += "3 ... VDD\n";
    s += "\n";
    s += "Connection to FORTE (parasite-powered mode):\n";
    s += "GND    <-> GND\n";
    s += "DQ     <-> P\n";
    s += "VDD    <-> GND\n";
    s += "\n";
    s += "Connection to FORTE (external supply mode):\n";
    s += "GND    <-> GND\n";
    s += "DQ     <-> P\n";
    s += "VDD    <-> VDD\n";
    return s;
}

int DeviceDS18B20::GetMemoriesCount()
{
    return 0;
}

void DeviceDS18B20::RegisterActions()
{
    AddAction(_T("Read"), _T("&Read\tF6"), (UserHandler)&DeviceDS18B20::Read);
    SetActionLayout(1);
}

// user-defined functions

void DeviceDS18B20::Read()
{
    if(!OpenProg()) {
        AddLog("Cannot open programmer.");
        return;
    }
    //
    int status1, status2, data_lo, data_hi, data;
    float temperature;
    wxString s;

    // set supply voltage to 3.3V
    prog->QPoweronVdd(20000, 3300); // supply voltage 3.3V
    ReadStatus("PoweronVdd");
    
    // enable 2k4 pull-up
    prog->QSetPullUpDowns((PULLUP << P_PULL));
    ReadStatus("SetPullUpDowns");
    
    // switch off the answer of the output GPIO functions and delay
    prog->QSetGPIOAnswer(0);
    ReadStatus("SetGPIOAnswer");
    
    // measure temperature
    prog->Q1WireInit();
    // skip ROM command
    prog->Q1WireWriteByte(0xCC, 0);
    // convert T command, with strong pull-up for 750ms, which is the conversion time
    prog->Q1WireWriteByte(0x44, 750000);    
    
    // read the measured value
    prog->Q1WireInit();
    // skip ROM command
    prog->Q1WireWriteByte(0xCC, 0);
    // read scratchpad, read only the two first bytes, which is the temperature
    prog->Q1WireWriteByte(0xBE, 0);
    prog->Q1WireReadByte();
    prog->Q1WireReadByte();

    // get results of the two inits
    status1 = prog->AGetBlocking();
    status2 = prog->AGetBlocking();
    // get the measured data
    data_lo = prog->AGetBlocking() & 0xFF;
    data_hi = prog->AGetBlocking() & 0xFF;

    if((status1 == _1WIRE_PRESENT) && (status2 == _1WIRE_PRESENT)) {
        data = (data_hi << 8) | data_lo;
        // convert to 16-bit signed
        if(data >= 0x8000) data = data - 0x10000;
        temperature = 0.0625*data;
        s.Printf("Temperature: %.1f degrees C", temperature);
        AddLog(s);
    }
    else {
        AddLog("1Wire: ERROR, device not present.");
    }

    // enable the GPIO answer
    prog->QSetGPIOAnswer(1);
    ReadStatus("SetGPIOAnswer");

    CloseProg();
}
