/////////////////////////////////////////////////////////////////////////////
// Name:        hexedit.h
// Purpose:     hexadecimal editor control
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

#ifndef HEX_EDIT_H
#define HEX_EDIT_H

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// header files
#include <wx/fontenum.h>
#include <wx/caret.h>
#include <wx/clipbrd.h>

//#define HEX_EDIT_H_DEBUG

// ----------------------------------------------------------------------------
// class HexEdit
// ----------------------------------------------------------------------------

typedef void (wxObject::*EditorUpdateHandler)(void);

class HexEdit: public wxScrolledWindow
{
public:
    // Constructor and destructor
    HexEdit(wxWindow *parent, wxWindowID id, int memory_word_count, int bytes_per_mem_word,
        const wxPoint &pos = wxDefaultPosition, const wxSize &size = wxDefaultSize, long style = wxVSCROLL);
	~HexEdit();
    // variables
    unsigned char *memory;
    // functions
    void Repaint(void);
    void ScrollWindow(int dx, int dy, const wxRect *prect) {};  // override method to stop the moving of DC bitmap (screen blinking)
    void SetCallback(wxObject* dst, EditorUpdateHandler handler);
    // event handlers
	void OnPaint(wxPaintEvent &event);
	void OnSize(wxSizeEvent &event);
	void OnChar(wxKeyEvent &event);
    void OnMouseLeft(wxMouseEvent& event);
	void OnMouseMove(wxMouseEvent& event);
	void OnFocus(wxFocusEvent& event);
    void OnScroll(wxScrollWinEvent &event);
private:
    // variables
    int word_count;
    int bytes_count;
    int bytes_per_word;
    int words_per_line;
    int bytes_per_line;
    int max_words_per_line;
    int address_digits;
    int address_offset; // words
    int address_offset_max;
    int cursor_pos; // nibbles
    bool cursor_in_hex;
    int sel_start; // nibbles
    int sel_stop; // nibbles
    bool selected;
    wxFont font;
    int character_width;
    int character_height;
    int visible_lines;
	wxMemoryDC* internalBufferDC;
	wxBitmap*   internalBufferBMP;
    bool update_callback_set;
    wxObject* update_dst;
    EditorUpdateHandler update_callback;
    // functions
    void Resize();
    wxMemoryDC* CreateDC(void);
    wxDC* UpdateDC();
    void MoveCaret();
    void PointToCoord(int x, int y, bool* in_hex, int* position);
    //
    void AdjustAddressOffset();
    void Left();
    void Right();
    void Up();
    void Down();
    void LineHome();
    void LineEnd();
    void FileHome();
    void FileEnd();
    void PageUp();
    void PageDown();
    //
    void ClipboardCopy();
    void ClipboardPaste(wxString text);
    void Write(wxString s, bool move_left=false);
    int ToHex(wxUniChar hex);
    wxChar GetChar(int pos); // bytes
    void MemoryChanged(void);

#ifdef HEX_EDIT_H_DEBUG
    void DebugMessage(wxString msg);
    void DebugMessage(int msg);
    void DebugMessage(int msg1, int msg2);
#endif // HEX_EDIT_H_DEBUG

    // any class wishing to process wxWidgets events must use this macro
    DECLARE_EVENT_TABLE()

};

#endif // HEX_EDIT_H