/////////////////////////////////////////////////////////////////////////////
// Name:        main.cpp
// Purpose:     main file of programmer demo
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// user header files
#include "string_const.h"
#include "programmer.h"
#include "mframe.h"

// ----------------------------------------------------------------------------
// private classes
// ----------------------------------------------------------------------------

// Define a new application type, each program should derive a class from wxApp
class MainApp : public wxApp
{
public:
    // override base class virtuals
    // ----------------------------
    // this one is called on application startup and is a good place for the app
    // initialization (doing it here and not in the ctor allows to have an error
    // return: if OnInit() returns false, the application terminates)
    virtual bool OnInit();
    int OnRun(void);
    int OnExit(void);
    // variables
    Programmer *prog;
    MainFrame *mfm;
    wxFileConfig *config;
private:
    //functions
    //
    // variables
    int app_return_value;
};

// Create a new application object: this macro will allow wxWidgets to create
// the application object during program execution (it's better than using a
// static object for many reasons) and also declares the accessor function
// wxGetApp() which will return the reference of the right type

IMPLEMENT_APP(MainApp)

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// class MainApp
// ----------------------------------------------------------------------------

// 'Main program' equivalent: the program execution "starts" here
bool MainApp::OnInit()
{
    // access to DLL
    int status;
    prog = new Programmer();
    status = prog->LoadDll();
    if(status != 0) {
        wxMessageBox(_T(STR_LOAD_DLL_ERROR), _T(STR_APP_CAPTION), wxICON_ERROR);
        app_return_value = 1;
        return true;
    }
    // read settings
    // usually stored in C:\Documents and Settings\username\Application Data\"STR_APP_NAME".ini
    config = new wxFileConfig(_T(STR_APP_NAME),wxEmptyString,wxEmptyString,wxEmptyString,wxCONFIG_USE_LOCAL_FILE);
    long win_x = config->ReadLong("/Main/PositionX", 0);
    long win_y = config->ReadLong("/Main/PositionY", 0);
    long win_w = config->ReadLong("/Main/Width", 1024);
    long win_h = config->ReadLong("/Main/Height", 768);
    // limits
    if(win_w < 50) win_w = 50;
    if(win_w > 5000) win_w = 5000;
    if(win_h < 40) win_h = 40;
    if(win_h > 4000) win_h = 4000;
    if(win_x > 10000) win_x = 0;
    if((win_x+win_w) < 0) win_x = 0;
    if(win_y > 10000) win_y = 0;
    if((win_y+win_h) < 0) win_y = 0;
    // create main window
    mfm = new MainFrame(config,prog,_T(STR_APP_CAPTION),wxPoint(win_x, win_y),wxSize(win_w,win_h),
        wxDEFAULT_FRAME_STYLE | wxFRAME_NO_WINDOW_MENU | wxHSCROLL | wxVSCROLL);
    mfm->Show(true);
    //
    app_return_value = 0;
    return true;
}

int MainApp::OnRun(void)
{
    if(app_return_value != 0) {
        ExitMainLoop();
        return app_return_value;
    }
    else {
        return wxApp::OnRun();
    }
}

int MainApp::OnExit(void)
{
    delete prog;
    delete config;   // the changes will be written automatically
    return wxApp::OnExit();
}

