/////////////////////////////////////////////////////////////////////////////
// Name:        mframe.cpp
// Purpose:     main programmer frame
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// standard header file
#include "mframe.h"

// ----------------------------------------------------------------------------
// Dynamic Arrays
// ----------------------------------------------------------------------------
#include <wx/arrimpl.cpp> // this is a magic incantation which must be done!

WX_DEFINE_OBJARRAY(ArrayOfDevices);

// ----------------------------------------------------------------------------
// event tables and other macros for wxWidgets
// ----------------------------------------------------------------------------

// the event tables connect the wxWidgets events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.

BEGIN_EVENT_TABLE(MainFrame, wxMDIParentFrame)
    EVT_CLOSE(MainFrame::OnCloseQuery)
    EVT_MENU(Menu_Quit,  MainFrame::OnQuit)
    EVT_MENU(Menu_About, MainFrame::OnAbout)
    EVT_MENU(Menu_ShowAll, MainFrame::OnShowAllClick)
    EVT_MENU(Menu_HideAll, MainFrame::OnHideAllClick)
    EVT_MENU(Menu_ShowAction, MainFrame::OnActionWindowShowClick)
    EVT_MENU(Menu_ShowBitAlias, MainFrame::OnBitAliasWindowShowClick)
    EVT_MENU(Menu_ShowDeviceHelp, MainFrame::OnDeviceHelpWindowShowClick)
    EVT_MENU(Menu_ShowLog, MainFrame::OnLogWindowShowClick)
    EVT_MENU(Menu_ShowSettings, MainFrame::OnSettingsWindowShowClick)

    EVT_MENU(Menu_New, MainFrame::OnNewFileClick)
    EVT_MENU(Menu_Open, MainFrame::OnOpenFileClick)
    EVT_MENU(Menu_SaveAs, MainFrame::OnSaveFileAsClick)

    EVT_MENU(Menu_ClearLog, MainFrame::OnLogClearClick)
    EVT_MENU(Menu_SaveLog, MainFrame::OnLogSaveClick)

    EVT_MENU(Menu_RefreshProgList, MainFrame::OnRefreshProgListClick)
END_EVENT_TABLE()

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// class MainFrame
// ----------------------------------------------------------------------------

// frame constructor
MainFrame::MainFrame(wxFileConfig *cfg, Programmer *prg, const wxString& title, const wxPoint& pos, const wxSize& size, long style)
       : wxMDIParentFrame(NULL, -1, title, pos, size, style)
{
    int i;
    config = cfg;
    prog = prg;
    // register device classes
    devices.Add(new Device24LC02());
    devices.Add(new Device93LC46B());
    devices.Add(new DeviceM25P80());
    devices.Add(new DeviceDS18B20());

    // other window layout
    this->Maximize(config->ReadBool("/Main/Maximized", false));
    SetIcon(wxICON(ICON_APP));
    CreateStatusBar(3);
    SetStatusText(_T(""),0);
    SetStatusText(_T(""),1);
    SetStatusText(_T(""),2);
    int statuswidths[3] = {-1, 100, 100};
    statuswidths[1] = config->ReadLong("/Main/StatusWidthProgrammer", statuswidths[1]);
    statuswidths[2] = config->ReadLong("/Main/StatusWidthDevice", statuswidths[2]);
    SetStatusWidths(3, statuswidths);
    // write back since it is not modified later
    config->Write("/Main/StatusWidthProgrammer", statuswidths[1]);
    config->Write("/Main/StatusWidthDevice", statuswidths[2]);

    //-------------------------------------------------
    // create a menu bar
    //-------------------------------------------------
    // File
    wxMenu *menuFile = new wxMenu;
    menuFile->Append(Menu_New, _T(STR_MENU_NEW), _T(STR_MENU_NEW_HINT));
    menuFile->Append(Menu_Open, _T(STR_MENU_OPEN), _T(STR_MENU_OPEN_HINT));
    menuFile->Append(Menu_SaveAs, _T(STR_MENU_SAVE), _T(STR_MENU_SAVE_HINT));
    menuFile->Append(Menu_ClearLog, _T(STR_MENU_CLEARLOG), _T(STR_MENU_CLEARLOG_HINT));
    menuFile->Append(Menu_SaveLog, _T(STR_MENU_SAVELOG), _T(STR_MENU_SAVELOG_HINT));
    menuFile->Append(Menu_SepFile1, _T(""), _T(""), wxITEM_SEPARATOR);
    
    // File-Device
    menuDevice = new wxMenu;
    for(i=0;i<(int)devices.Count();i++) {
        menuDevice->Append(Menu_DeviceList+i, devices[i].GetName(), devices[i].GetShortHint(), wxITEM_RADIO);
    }
    Connect(Menu_DeviceList, Menu_DeviceList+devices.Count()-1, wxEVT_MENU, wxCommandEventHandler(MainFrame::OnDeviceSelect));
    menuFile->Append(Menu_Device, _T(STR_MENU_DEVICE), menuDevice, _T(STR_MENU_DEVICE_HINT));

    // File-Programmer
    menuProg = new wxMenu;
    menuProg->Append(Menu_RefreshProgList, _T(STR_MENU_PROG_REFRESH), _T(STR_MENU_PROG_REFRESH_HINT));
    menuProg->Append(Menu_SepProg1, _T(""), _T(""), wxITEM_SEPARATOR);
    menuProg->Append(Menu_ProgList, _T(STR_MENU_PROG_DEFAULT), _T(""), wxITEM_RADIO);
    Connect(Menu_ProgList, wxEVT_MENU, wxCommandEventHandler(MainFrame::OnProgrammerClick));
    menuFile->Append(Menu_Prog, _T(STR_MENU_PROG), menuProg, _T(STR_MENU_PROG_HINT));

    // File
    menuFile->Append(Menu_SepFile2, _T(""), _T(""), wxITEM_SEPARATOR);
    menuFile->Append(Menu_Quit, _T(STR_MENU_EXIT), _T(STR_MENU_EXIT_HINT));

    // Action
    menuAction = new wxMenu;

    // Window
    menuWindow = new wxMenu;
    menuWindow->Append(Menu_ShowAll, _T(STR_MENU_SHOW_ALL));
    menuWindow->Append(Menu_HideAll, _T(STR_MENU_HIDE_ALL));
    menuWindow->Append(Menu_SepWindow1, _T(""), _T(""), wxITEM_SEPARATOR);
 
    // Help
    // the "About" item should be in the help menu
    wxMenu *helpMenu = new wxMenu;
    helpMenu->Append(Menu_About, _T(STR_MENU_ABOUT), _T(STR_MENU_ABOUT_HINT));

    // now append the freshly created menu to the menu bar...
    wxMenuBar *menuBar = new wxMenuBar();
    menuBar->Append(menuFile, _T(STR_MENU_FILE));
    menuBar->Append(menuAction, _T(STR_MENU_ACTION));
    menuBar->Append(menuWindow, _T(STR_MENU_WINDOW));
    menuBar->Append(helpMenu, _T(STR_MENU_HELP));
    // ... and attach this menu bar to the frame
    SetMenuBar(menuBar);

    //-------------------------------------------------
    // Startup
    //-------------------------------------------------
    
    // load device
    wxString startup_device = config->Read("/Main/Device",  _T(""));
    int device_index = 0;
    for(i=0;i<(int)devices.Count();i++) {
        if(devices[i].GetName() == startup_device) {
            device_index = i;
            break;
        }
    }
    SelectDevice(device_index);

    // programmer
    prog->preferred_prog_sn = config->ReadLong("/Main/ProgrammerSN", -1);
    RefreshProgList();
    
}

void MainFrame::SelectDevice(int device_index)
{
    current_device = &(devices[device_index]);
    wxMenuItem* device_in_menu = menuDevice->FindItemByPosition(device_index);
    device_in_menu->Check();
    SetStatusText(current_device->GetName(),2);
    current_device->Open(this, config, menuAction, menuWindow, prog);
	if(current_device->GetMemoriesCount() > 0) {
		Connect(Menu_MemoryList, Menu_MemoryList + current_device->GetMemoriesCount()-1, wxEVT_MENU,
			wxCommandEventHandler(MainFrame::OnMemoryWindowShowClick));
	}
    if(current_device->GetActionCount()>0) {
        Connect(Menu_ActionList, Menu_ActionList+current_device->GetActionCount()-1, wxEVT_MENU,
            wxCommandEventHandler(MainFrame::OnActionClick));
        Connect(Menu_ActionList, Menu_ActionList+current_device->GetActionCount()-1, wxEVT_BUTTON,
            wxCommandEventHandler(MainFrame::OnActionClick));
    }
    if(current_device->GetBitAliasCount()>0) {
        Connect(Menu_BitAlias, wxEVT_PG_CHANGED, wxPropertyGridEventHandler(MainFrame::OnBitAliasWindowValueChanged));
        current_device->RefreshScreen();
    }
}

void MainFrame::DeselectDevice()
{
    SetStatusText(_T(""),2);
	if(current_device->GetMemoriesCount() > 0) {
		Disconnect(Menu_MemoryList, Menu_MemoryList + current_device->GetMemoriesCount()-1, wxEVT_MENU);
	}
    if(current_device->GetActionCount()>0) {
        Disconnect(Menu_ActionList, Menu_ActionList+current_device->GetActionCount()-1, wxEVT_MENU);
        Disconnect(Menu_ActionList, Menu_ActionList+current_device->GetActionCount()-1, wxEVT_BUTTON);
    }
    if(current_device->GetBitAliasCount()>0) {
        Disconnect(Menu_BitAlias, wxEVT_PG_CHANGED);
    }
    current_device->Close();
}


// event handlers

void MainFrame::OnCloseQuery(wxCloseEvent& event)
{
    if(1) {
        wxRect win_xywh = this->GetRect();
        config->Write("/Main/PositionX", win_xywh.x);
        config->Write("/Main/PositionY", win_xywh.y);
        config->Write("/Main/Width", win_xywh.width);
        config->Write("/Main/Height", win_xywh.height);
        config->Write("/Main/Maximized", this->IsMaximized());
        config->Write("/Main/Device", current_device->GetName());
        config->Write("/Main/ProgrammerSN", prog->preferred_prog_sn);
        DeselectDevice();
        event.Skip();
    }
    else {
        //controlbox->OnMessage(zcEVT_CLOSEQUERY, NULL, NULL);
        event.Veto();
    }
}

void MainFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    Close(TRUE);    // TRUE is to force the frame to close
}

void MainFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
    wxMessageBox(_T(STR_ABOUT_MESSAGE), _T(STR_ABOUT_CAPTION), wxOK | wxICON_INFORMATION, this);
}

void MainFrame::OnDeviceSelect(wxCommandEvent& evt)
{
    int id = evt.GetId();
    DeselectDevice();
    SelectDevice(id-Menu_DeviceList);
}

void MainFrame::OnShowAllClick(wxCommandEvent& evt)
{
    current_device->ShowAll();
}

void MainFrame::OnHideAllClick(wxCommandEvent& evt)
{
    current_device->HideAll();
}

void MainFrame::OnActionWindowShowClick(wxCommandEvent& evt)
{
    int id = evt.GetId();
    wxMenuItem *mi = menuWindow->FindChildItem(id);
    bool checked = mi->IsChecked();
    current_device->ShowActionWindow(checked);
}

void MainFrame::OnBitAliasWindowShowClick(wxCommandEvent& evt)
{
    int id = evt.GetId();
    wxMenuItem *mi = menuWindow->FindChildItem(id);
    bool checked = mi->IsChecked();
    current_device->ShowBitAliasWindow(checked);
}

void MainFrame::OnDeviceHelpWindowShowClick(wxCommandEvent& evt)
{
    int id = evt.GetId();
    wxMenuItem *mi = menuWindow->FindChildItem(id);
    bool checked = mi->IsChecked();
    current_device->ShowDeviceHelpWindow(checked);
}

void MainFrame::OnLogWindowShowClick(wxCommandEvent& evt)
{
    int id = evt.GetId();
    wxMenuItem *mi = menuWindow->FindChildItem(id);
    bool checked = mi->IsChecked();
    current_device->ShowLogWindow(checked);
}

void MainFrame::OnSettingsWindowShowClick(wxCommandEvent& evt)
{
    int id = evt.GetId();
    wxMenuItem *mi = menuWindow->FindChildItem(id);
    bool checked = mi->IsChecked();
    current_device->ShowSettingsWindow(checked);
}

void MainFrame::OnMemoryWindowShowClick(wxCommandEvent& evt)
{
    int id = evt.GetId();
    wxMenuItem *mi = menuWindow->FindChildItem(id);
    bool checked = mi->IsChecked();
    current_device->ShowMemoryWindow(id-Menu_MemoryList, checked);
}

void MainFrame::OnActionClick(wxCommandEvent& evt)
{
    int id = evt.GetId();
    current_device->Action(id-Menu_ActionList);
}

void MainFrame::OnBitAliasWindowValueChanged(wxPropertyGridEvent& evt)
{
    current_device->BitAliasWindowToMemoryWindows(evt.GetProperty());
}

void MainFrame::OnNewFileClick(wxCommandEvent& evt)
{
    current_device->NewFile();
}

void MainFrame::OnOpenFileClick(wxCommandEvent& evt)
{
    current_device->OpenFile();
}

void MainFrame::OnSaveFileAsClick(wxCommandEvent& evt)
{
    current_device->SaveFileAs();
}

void MainFrame::OnLogClearClick(wxCommandEvent& evt)
{
    current_device->ClearLog();
}

void MainFrame::OnLogSaveClick(wxCommandEvent& evt)
{    
    wxString filename = wxFileSelector(STR_SAVE_LOG_CAPTION, wxEmptyString, wxEmptyString, wxEmptyString,
        STR_SAVE_LOG_FILTER, wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(!filename.empty()) {
        current_device->SaveLog(filename);
    }
}

void MainFrame::OnRefreshProgListClick(wxCommandEvent& evt)
{
    RefreshProgList();
}

void MainFrame::OnProgrammerClick(wxCommandEvent& evt)
{
    int id = evt.GetId();
    if(id == Menu_ProgList) {
        prog->preferred_prog_sn = -1;
    }
    else {
        prog->preferred_prog_sn = prog_list[id - Menu_ProgList - 1];
    }
    wxMenuItem *mi = menuProg->FindChildItem(id);
    SetStatusText(mi->GetItemLabelText(),1);
}

void MainFrame::RefreshProgList()
{
    #define MAX_PROG_COUNT 100
    int i;
    int sn_list[MAX_PROG_COUNT];
    int count;
    wxString buf;
    // clear existing menu
    count = prog_list.Count();
    for(i=0;i<count;i++) {
        wxMenuItem *mi = menuProg->FindChildItem(Menu_ProgList+i+1); // do not delete "Default"
        menuProg->Remove(mi);
        Disconnect(Menu_ProgList+i+1, wxEVT_MENU);
    }
    prog_list.Clear();
    // get current list
    prog->AGetProgList(sn_list, MAX_PROG_COUNT, &count);
    for(i=0;i<count;i++) {
        prog_list.Add(sn_list[i]);
        buf.Printf("S/N %06X", sn_list[i]);
        menuProg->Append(Menu_ProgList+i+1, buf, _T(""), wxITEM_RADIO);
        Connect(Menu_ProgList+i+1, wxEVT_MENU, wxCommandEventHandler(MainFrame::OnProgrammerClick));
    }
    // check / uncheck
    count = prog_list.Count();
    bool check_default = true;
    for(i=0;i<count;i++) {
        wxMenuItem *mi = menuProg->FindChildItem(Menu_ProgList+i+1);
        if(prog->preferred_prog_sn == prog_list[i]) {
            mi->Check(true);
            SetStatusText(mi->GetItemLabelText(),1);
            check_default = false;
        }
        else {
            mi->Check(false);
        }
    }
    wxMenuItem *mi = menuProg->FindChildItem(Menu_ProgList);
    if(check_default) {
        mi->Check(true);
        SetStatusText(mi->GetItemLabelText(),1);
        if(prog->preferred_prog_sn != -1) {
            buf.Printf(STR_PROG_SN_WARNING, prog->preferred_prog_sn);
            current_device->AddLog(buf);
        }
        prog->preferred_prog_sn = -1;
    }
    else {
        mi->Check(false);
    }
}
