#
# This file is python wrapper for ASIX FORTE programmer DLL
# distributed as-is, without any warranties
# http://www.asix.net/
#
# version 1.0, Dec 2016
#

from ctypes import * 

# QSetPins constants
PINS_HIZ = 0x01;
PINS_LO = 0x02;
PINS_HI = 0x03;
PINS_D_BIT = 0x00;
PINS_C_BIT = 0x02;
PINS_I_BIT = 0x04;
PINS_L_BIT = 0x06;
PINS_P_BIT = 0x08;
PINS_R_BIT = 0x0A;
PINS_S_BIT = 0x0C;
PINS_T_BIT = 0x0E;

# QShift... constants
SHIFT_OUTIN_PIND = 0x00;
SHIFT_OUTIN_PINI = 0x02;
SHIFT_OUTIN_PINL = 0x03;
SHIFT_OUTIN_PINP = 0x04;
SHIFT_OUTIN_PINR = 0x05;
SHIFT_OUTIN_PINS = 0x06;
SHIFT_OUTIN_PINT = 0x07;
SHIFT_MODE0 = 0x00;
SHIFT_MODE1 = 0x01;
SHIFT_MODE2 = 0x02;
SHIFT_MODE3 = 0x03;

# QSetShiftSpeed constants
SHIFT_CLK_15000kHz = 1;
SHIFT_CLK_10000kHz = 2;
SHIFT_CLK_7500kHz = 3;
SHIFT_CLK_6000kHz = 4;
SHIFT_CLK_5000kHz = 5;
SHIFT_CLK_3750kHz = 6;
SHIFT_CLK_3330kHz = 7;
SHIFT_CLK_3000kHz = 8;
SHIFT_CLK_2500kHz = 9;
SHIFT_CLK_2000kHz = 10;
SHIFT_CLK_1500kHz = 11;
SHIFT_CLK_1000kHz = 12;
SHIFT_CLK_750kHz = 13;
SHIFT_CLK_600kHz = 14;
SHIFT_CLK_500kHz = 15;
SHIFT_CLK_400kHz = 16;
SHIFT_CLK_375kHz = 17;
SHIFT_CLK_333kHz = 18;
SHIFT_CLK_300kHz = 19;
SHIFT_CLK_250kHz = 20;
SHIFT_CLK_200kHz = 21;
SHIFT_CLK_150kHz = 22;
SHIFT_CLK_120kHz = 23;
SHIFT_CLK_100kHz = 24;
SHIFT_CLK_75kHz = 25;
SHIFT_CLK_60kHz = 26;
SHIFT_CLK_50kHz = 27;
SHIFT_CLK_40kHz = 28;
SHIFT_CLK_37kHz = 29;
SHIFT_CLK_33kHz = 30;
SHIFT_CLK_30kHz = 31;

# QSetPullUpDowns constants
PULLDOWN = 0x01;
PULLUP = 0x02;
D_PULL = 0x00;
C_PULL = 0x02;
I_PULL = 0x04;
L_PULL = 0x06;
S_PULL = 0x08;
T_PULL = 0x0A;
P_PULL = 0x0C;
R_PULL = 0x0E;

# QI2CSetSpeed constants
I2C_CLK_100kHz = 0x00;
I2C_CLK_400kHz = 0x01;
I2C_CLK_1MHz = 0x02;

# QSetActiveLED constants
LED_ACT_OFF = 0x00;
LED_ACT_Y = 0x01;
LED_ACT_R = 0x02;
LED_ACT_Y_BLINK = 0x03;
LED_ACT_R_BLINK = 0x04;
LED_ACT_Y_FAST_BLINK = 0x05;
LED_ACT_R_FAST_BLINK = 0x06;
LED_ACT_YR_FAST_BLINK = 0x07;

# return codes
OPEN_OK = 0x10000;
OPEN_NOTFOUND = 0x10001;
OPEN_CANNOTOPEN = 0x10002;
OPEN_ALREADYOPEN = 0x10003;
OPEN_BADDRIVERVERSION = 0x10004;
CLOSE_OK = 0x20000;
CLOSE_CANNOTCLOSE = 0x20001;
POWERON_OK = 0x30000;
POWERON_OCURR = 0x30001;
POWERON_WRONG_LEVEL = 0x30002;
GETPINS_CODE = 0x40000; # ORed with GETPINS_PINx
GETPINS_PIND = 0x01;
GETPINS_PINC = 0x02;
GETPINS_PINI = 0x04;
GETPINS_PINL = 0x08;
GETPINS_PINP = 0x10;
GETPINS_PINR = 0x20;
GETPINS_PINS = 0x40;
GETPINS_PINT = 0x80;
OK = 0x50000;
NOT_OPENED = 0x50001;
WRONG_INPUT = 0x50002;
SHIFT_BYTE_OUTIN_CODE = 0x60000;
SUPPLY_VOLTAGE_CODE = 0x70000;
VPP_OK = 0x80000;
VPP_OCURR = 0x80001;
VPP_WRONG_LEVEL = 0x80002;
GO_BUTTON_NOT_PRESSED = 0x90000;
GO_BUTTON_PRESSED = 0x90001;
SHIFT_BITS_OUTIN_CODE = 0xA0000;
ONE_WIRE = 0xB0000;
ONE_WIRE_PRESENT = 0xB0100;
ONE_1WIRE_NOT_PRESENT = 0xB0200;
I2C_CODE = 0xC0000;
I2C_ACK = 0xC0100;
I2C_NACK = 0xC0200;
FATAL_OVERCURRENTVDD = 0x01;
FATAL_OVERCURRENTVPP = 0x02;
FATAL_OVERVOLTAGEVDD = 0x04;
FATAL_OTHER = 0x08;

# reverse return code strings
RETURNCODE = {
0x10000: "OPEN_OK",
0x10001: "OPEN_NOTFOUND",
0x10002: "OPEN_CANNOTOPEN",
0x10003: "OPEN_ALREADYOPEN",
0x10004: "OPEN_BADDRIVERVERSION",
0x20000: "CLOSE_OK",
0x20001: "CLOSE_CANNOTCLOSE",
0x30000: "POWERON_OK",
0x30001: "POWERON_OCURR",
0x30002: "POWERON_WRONG_LEVEL",
0x40000: "GETPINS_CODE",
0x50000: "OK",
0x50001: "NOT_OPENED",
0x50002: "WRONG_INPUT",
0x60000: "SHIFT_BYTE_OUTIN_CODE",
0x70000: "SUPPLY_VOLTAGE_CODE",
0x80000: "VPP_OK",
0x80001: "VPP_OCURR",
0x80002: "VPP_WRONG_LEVEL",
0x90000: "GO_BUTTON_NOT_PRESSED",
0x90001: "GO_BUTTON_PRESSED",
0xA0000: "SHIFT_BITS_OUTIN_CODE",
0xB0000: "ONE_WIRE",
0xB0100: "ONE_WIRE_PRESENT",
0xB0200: "ONE_1WIRE_NOT_PRESENT",
0xC0000: "I2C_CODE",
0xC0100: "I2C_ACK",
0xC0200: "I2C_NACK",
0x01: "FATAL_OVERCURRENTVDD",
0x02: "FATAL_OVERCURRENTVPP",
0x04: "FATAL_OVERVOLTAGEVDD",
0x08: "FATAL_OTHER"
}

# load DLL library
def LoadForteDll(filename="forte.dll"):
    global fortedll
    fortedll = WinDLL(filename)

# exported functions

# void __stdcall AClearFatalError(void);
def AClearFatalError():
    fortedll.AClearFatalError()

# bool __stdcall AGet(int *answer);
def AGet():
    answer_c = c_int()
    status = fortedll.AGet(byref(answer_c))
    return bool(status), answer_c.value

# bool __stdcall AGetBlock(int *buf, int count, int *count_returned);
def AGetBlock(count):
    count_c = c_int(count)
    buf_type = c_int * count
    buf_c = buf_type()
    count_returned_c = c_int()
    status = fortedll.AGetBlock(buf_c, count_c, byref(count_returned_c))
    count_returned = count_returned_c.value
    buf = [0] * count_returned
    for i in range(count_returned):
        buf[i] = buf_c[i]
    return buf

# int __stdcall AGetBlocking(void);
def AGetBlocking():
    return int(fortedll.AGetBlocking())

# void __stdcall AGetProgList(int *sn_list, int count, int *count_returned);
def AGetProgList(count=100):
    count_c = c_int(count)
    snlist_type = c_int * count
    snlist_c = snlist_type()
    count_returned_c = c_int()
    fortedll.AGetProgList(snlist_c, count_c, byref(count_returned_c))
    count_returned = count_returned_c.value
    snlist = [0] * count_returned
    for i in range(count_returned):
        snlist[i] = snlist_c[i]
    return snlist

# bool __stdcall AGetStatus(int *NumberOfAnswers);
def AGetStatus():
    NumberOfAnswers_c = c_int()
    status = fortedll.AGetStatus(byref(NumberOfAnswers_c))
    return bool(status), NumberOfAnswers_c.value

# void __stdcall Q1WireInit(void);
def Q1WireInit():
    fortedll.Q1WireInit()

# void __stdcall Q1WireReadByte(void);
def Q1WireReadByte():
    fortedll.Q1WireReadByte()

# void __stdcall Q1WireWriteByte(int data, int strong_pullup_time_us);
def Q1WireWriteByte(data, strong_pullup_time_us):
    data_c = c_int(data)
    strong_pullup_time_us_c = c_int(strong_pullup_time_us)
    fortedll.Q1WireWriteByte(data_c, strong_pullup_time_us_c)

# void __stdcall QCheckGoButton(void);
def QCheckGoButton():
    fortedll.QCheckGoButton()

# void __stdcall QCheckSupplyVoltage(void);
def QCheckSupplyVoltage():
    fortedll.QCheckSupplyVoltage()

# void __stdcall QCloseProg(void);
def QCloseProg():
    fortedll.QCloseProg()

# void __stdcall QDelay(int delayus);
def QDelay(delayus):
    delayus_c = c_int(delayus)
    fortedll.QDelay(delayus_c)

# void __stdcall QDelay_ns(int delayns);
def QDelay_ns(delayns):
    delayns_c = c_int(delayns)
    fortedll.QDelay_ns(delayns_c)

# void __stdcall QGetPins(void);
def QGetPins():
    fortedll.QGetPins()

# void __stdcall QI2CReadByte(bool ACK);
def QI2CReadByte(ACK):
    ACK_c = c_bool(ACK)
    fortedll.QI2CReadByte(ACK_c)

# void __stdcall QI2CSetSpeed(int speed);
def QI2CSetSpeed(speed):
    speed_c = c_int(speed)
    fortedll.QI2CSetSpeed(speed_c)

# void __stdcall QI2CStart(bool UseIntPullUps);
def QI2CStart(UseIntPullUps):
    UseIntPullUps_c = c_bool(UseIntPullUps)
    fortedll.QI2CStart(UseIntPullUps_c)

# void __stdcall QI2CStop(void);
def QI2CStop():
    fortedll.QI2CStop()

# void __stdcall QI2CWriteByte(int databyte);
def QI2CWriteByte(databyte):
    databyte_c = c_int(databyte)
    fortedll.QI2CWriteByte(databyte_c)

# void __stdcall QOpenProg(int sn);
def QOpenProg(sn = -1):
    sn_c = c_int(sn)
    fortedll.QOpenProg(sn_c)

# void __stdcall QPoweroffVdd(void);
def QPoweroffVdd():
    fortedll.QPoweroffVdd()

# void __stdcall QPoweroffVpp(void);
def QPoweroffVpp():
    fortedll.QPoweroffVpp()

# void __stdcall QPoweronVdd(int delayus, int Voltage_mV);
def QPoweronVdd(delayus, Voltage_mV):
    delayus_c = c_int(delayus)
    Voltage_mV_c = c_int(Voltage_mV)
    fortedll.QPoweronVdd(delayus_c, Voltage_mV_c)

# void __stdcall QPoweronVpp(int Voltage_mV);
def QPoweronVpp(Voltage_mV):
    Voltage_mV_c = c_int(Voltage_mV)
    fortedll.QPoweronVpp(Voltage_mV_c)

# void __stdcall QSetActiveLED(int led);
def QSetActiveLED(led):
    led_c = c_int(led)
    fortedll.QSetActiveLED(led_c)

# void __stdcall QSetGPIOAnswer(bool answer);
def QSetGPIOAnswer(answer):
    answer_c = c_int(answer)
    fortedll.QSetGPIOAnswer(answer_c)

# void __stdcall QSetPins(int pins);
def QSetPins(pins):
    pins_c = c_int(pins)
    fortedll.QSetPins(pins_c)

# void __stdcall QSetPullUpDowns(int pullupdowns);
def QSetPullUpDowns(pullupdowns):
    pullupdowns_c = c_int(pullupdowns)
    fortedll.QSetPullUpDowns(pullupdowns_c)

# void __stdcall QSetShiftSpeed(int speed);
def QSetShiftSpeed(speed):
    speed_c = c_int(speed)
    fortedll.QSetShiftSpeed(speed_c)

# void __stdcall QShiftBits(int data, int mode, int bits_count);
def QShiftBits(data, mode, bits_count):
    data_c = c_int(data)
    mode_c = c_int(mode)
    bits_count_c = c_int(bits_count)
    fortedll.QShiftBits(data_c, mode_c, bits_count_c)

# void __stdcall QShiftBits_OutIn(int data, int mode, int InputPin, int bits_count);
def QShiftBits_OutIn(data, mode, InputPin, bits_count):
    data_c = c_int(data)
    mode_c = c_int(mode)
    InputPin_c = c_int(InputPin)
    bits_count_c = c_int(bits_count)
    fortedll.QShiftBits_OutIn(data_c, mode_c, InputPin_c, bits_count_c)

# void __stdcall QShiftByte(int databyte, int mode);
def QShiftByte(databyte, mode):
    databyte_c = c_int(databyte)
    mode_c = c_int(mode)
    fortedll.QShiftByte(databyte_c, mode_c)

# void __stdcall QShiftByte_OutIn(int databyte, int mode, int InputPin);
def QShiftByte_OutIn(databyte, mode, InputPin):
    databyte_c = c_int(databyte)
    mode_c = c_int(mode)
    InputPin_c = c_int(InputPin)
    fortedll.QShiftByte_OutIn(databyte_c, mode_c, InputPin_c)

# void __stdcall QShiftBytes(int *buf, int mode, int count);
def QShiftBytes(databytes, mode):
    count = len(databytes)
    buf_type = c_int * count
    buf_c = buf_type()
    for i in range(count):
        buf_c[i] = databytes[i]
    mode_c = c_int(mode)
    count_c = c_int(count)
    fortedll.QShiftBytes(buf_c, mode_c, count_c)

# void __stdcall QShiftBytes_OutIn(int *buf, int mode, int InputPin, int count); 
def QShiftBytes_OutIn(databytes, mode, InputPin):
    count = len(databytes)
    buf_type = c_int * count
    buf_c = buf_type()
    for i in range(count):
        buf_c[i] = databytes[i]
    mode_c = c_int(mode)
    InputPin_c = c_int(InputPin)
    count_c = c_int(count)
    fortedll.QShiftBytes_OutIn(buf_c, mode_c, InputPin_c, count_c)

# void __stdcall QShiftBytes_In(int mode, int InputPin, int count);
def QShiftBytes_In(mode, InputPin, count):
    mode_c = c_int(mode)
    InputPin_c = c_int(InputPin)
    count_c = c_int(count)
    fortedll.QShiftBytes_In(mode_c, InputPin_c, count_c)


