#
# ASIX FORTE I2C serial EEPROM 24LC02 example
# distributed as-is, without any warranties
# http://www.asix.net/
#
# version 1.0, Dec 2016
#
# Programmer connection table: 
# 24LC02  <->  FORTE
# A0      <->  GND
# A1      <->  GND
# A2      <->  GND
# VSS     <->  GND
# SDA     <->  D
# SCL     <->  C
# WP      <->  GND
# VCC     <->  VDD
#

from forte import *

def ReadStatus(message = ""):
    status = AGetBlocking()
    if message != "":
        print(message + ": " + RETURNCODE[status])
    else:
        print(RETURNCODE[status])
        
def PrintHex(data):
    for j in range(len(data)/16):
        s = ""
        for i in range(16):
            s += "%02X "%(data[j*16+i])
        print(s)
    j = len(data)/16
    s = ""
    for i in range(len(data)%16):
        s += "%02X "%(data[j*16+i])
    if s != "":
        print(s)

def main():
    LoadForteDll()
    QOpenProg()
    ReadStatus()
    
    # set supply voltage to 3.3V
    QPoweronVdd(20000, 3300)
    ReadStatus("PoweronVdd")
    # 100kHz
    QI2CSetSpeed(I2C_CLK_100kHz)
    ReadStatus("I2CSetSpeed")
    
    # set word address
    QI2CStart(True)
    ReadStatus("I2CStart")
    QI2CWriteByte(0xA0)
    ReadStatus("I2C control byte")
    QI2CWriteByte(0x00)
    ReadStatus("I2C word address")
    # read memory
    count = 256
    QI2CStart(True)
    ReadStatus("I2CStart")
    QI2CWriteByte(0xA1)
    ReadStatus("I2C control byte")
    for i in range(count-1):
        QI2CReadByte(True)
    QI2CReadByte(False)
    QI2CStop()
    # get data
    while True:
        status, received_bytes = AGetStatus()
        if not status:
            data = []
            break
        if received_bytes >= count:
            data = AGetBlock(count)
            break
    # take 8-bit data and print
    for i in range(len(data)):
        data[i] = data[i] & 0xFF
    print("Data:")
    PrintHex(data)
    
    ReadStatus("I2CStop")
    
    QCloseProg()
    ReadStatus()


if __name__ == "__main__":
    main()
